function [C,H] = build_cauchy_second_derivative_matrix_boundary(z, dens);

H = build_sampling_matrix(z, dens, 0, 0);
ze = H*z;

m = length(ze);
n = length(z);
C = zeros(m,n);

for ii=1:n
    ip1 = ii+1; if ip1 > n ip1 = 1; end;
    im1 = ii-1; if im1 == 0 im1 = n; end;
    zim1 = z(im1); zi = z(ii); zip1 = z(ip1);

    Bim1 = zim1 - ze;
    Bi = zi - ze;
    Bip1 = zip1 - ze;    

    C(:,ii) = (1./(Bim1.*Bi) - 1./(Bi.*Bip1))/(2*pi*1i);
end

function H = build_sampling_matrix(z, dens, st, en)

n = length(z);

I = [];
J = [];
S = [];
for j=1:n
    jp1 = j+1; if jp1 > n jp1 = 1; end;
    l = abs(z(jp1)-z(j));
    ns = round(l*dens);
    t = linspace(0,1,ns+1);
    if st == 0
        t = t(2:end);
    end
    if en == 0
        t = t(1:end-1);
    end
    t = t';
       
    ni = 0; if length(I) > 0 ni = I(end); end;
    In = [ni+1:ni+length(t)]';
    Jn = repmat(j,length(In),1);
    Jnp1 = repmat(jp1,length(In),1);
    Sn = 1-t;
    Snp1 = t;
    
    I = [I;In;In];
    J = [J;Jn;Jnp1];
    S = [S;Sn;Snp1];
end

H = sparse(I,J,S,I(end),n);

    
    
    
