clear all
close all
addpath('models');

% load model
filename = 'blue_monster';
[X,T] = read_off(filename);
X2D = X(:,1:2);
patch('Faces',T,'Vertices',X2D,'FaceColor','w'); axis equal; hold on;
aa = axis; aa = aa + 0.1*[-1,1,-1,1]; axis(aa);

% load cage or draw one (click for points, double click to finish)
if ~exist([filename '_cage.mat'])
    [x,y] = getline(gcf,'closed');
    x = x(1:end-1); y = y(1:end-1);
    save(['models\' filename '_cage'],'x','y');
 else
    S = load(['models\' filename '_cage']);
    x = S.x; y = S.y;
end
drawPolygon(x,y);

clear i;
z = x + 1i*y;
n = length(z);
drawPolygonC(z);

% load pinned points or pick some (click on npinned points)
if ~exist([filename '_pins.mat'])
    npinned = 8;
    pinned = [];
    for j=1:npinned
        l = pick_vertex(X2D);
        pinned(end+1,1) = l;
        draw_point_2d(X2D(l,:),'MarkerSize',30,'color','b');
    end
    save([filename '_pins.mat'], 'pinned');
else
    S = load([filename '_pins.mat']);
    pinned = S.pinned;
end
    
xpinned = X2D(pinned,:);

% Build coordinates
zt = X2D(:,1) + 1i*X2D(:,2);
C = build_cauchy_matrix(zt, z);
zp = zt(pinned);

% Build second derivative coordinates for P2P
dens = 100;
[Cbd2,H] = build_cauchy_second_derivative_matrix_boundary(z, dens);

npinned = length(zp);

% Build coordinates for pinned points
Cp = build_cauchy_matrix(zp, z);

% Build N (see Section 3.2)
wei = 0.001;
N = pinv([5000*Cp;wei*Cbd2]);
N = N(:,1:npinned);

% Show figure where interactive movement is enabled - on the right hand
% side figure, pick points by clicking and dragging. The figure will move
% accordingly. The source appears on the left.
interactive_move_p2p(T,z,zt,zp,C,Cp,N)